# include Interface
source mcPolymerInterface.tcl

# define low molecular species
# MMA = M1
Monomer M1
# Styrene = M2
Monomer M2
# alkoxyamine
Species RN
# 2-(hydroxycarbonyl)prop-2-yl radical
Species R#
# SG1
Species N#
# temination product
Species RR
# radical adducts
Species R-M1#
Species R-M2#
# termination products
Species R-M1-M1-R
Species R-M1-M2-R
Species R-M2-M2-R
# reversible combination products
Species R-M1N
Species R-M2N
# hydroxylamine
Species NH

# define macromolecules
# copolymers
SpeciesMacroCopolymer P-M1-M1#
SpeciesMacroCopolymer P-M2-M1#
SpeciesMacroCopolymer P-M1-M2#
SpeciesMacroCopolymer P-M2-M2#
# sleeping copolymers
SpeciesMacroCopolymer P-M1-M1N
SpeciesMacroCopolymer P-M2-M1N
SpeciesMacroCopolymer P-M1-M2N
SpeciesMacroCopolymer P-M2-M2N
# dead copolymer
SpeciesMacroCopolymerD D
SpeciesMacroCopolymer DbHtr

Concentration M1 8.48
Concentration M2 0.82
Concentration RN 0.0273
Concentration N# 0.00274

# debug function
ListAllSpecies  

# reaction coefficients
RateConstant kd0 3.4e-2
RateConstant kc0 5.0e+6
RateConstant kt0 2.0e+9
RateConstant kp01 1.9e+4
RateConstant kp02 2.56e+4

RateConstant kcR1 1.1e+6
RateConstant kcR2 4.6e+6
RateConstant kdR1 1.17
RateConstant kdR2 3.4e-3
RateConstant ktR1R1 2.0e+9
RateConstant ktR1R2 2.0e+9
RateConstant ktR2R2 2.0e+9
RateConstant kpR11 1.64e+3
RateConstant kpR12 3.33e+3
RateConstant kpR21 6.65e+2 
RateConstant kpR22 3.25e+2

RateConstant kp111 1.64e+3
RateConstant kp222 9.0e+2
RateConstant kp112 3.33e+3
RateConstant kp121 6.65e+2
RateConstant kp122 3.25e+2
RateConstant kp211 9.86e+2
RateConstant kp212 2.0e+3
RateConstant kp221 1.84e+3
RateConstant kc1 1.4e+4
RateConstant kc2 4.5e+5
RateConstant kd11 1.17
RateConstant kd12 3.4e-3
RateConstant kd21 6.9e-2
RateConstant kd22 2.5e-4
RateConstant ktd11 2.8e+7
RateConstant ktd12 1.8e+7
RateConstant ktd22 1.9e+6
RateConstant ktc11 6.5e+6
RateConstant ktc12 1.8e+7
RateConstant ktc22 3.3e+7
RateConstant kbHtr 1.69e+3

# debug function
ListAllRateConstants

# reactions 
# Homolytic dissociation of the alkoxyamine
Elemental_1E2P RN --> R# + N# kd0
# Recombination of the alkoxyamine
Elemental_2E1P R# + N# --> RN kc0
# Homotermination of the 2-(hydroxycarbonyl)prop-2-yl radical
Elemental_2E1P R# + R# --> RR kt0
# Addition of the 2-(hydroxycarbonyl)prop-2-yl radical onto M1
Elemental_2E1P R# + M1 --> R-M1# kp01
# Addition of the 2-(hydroxycarbonyl)prop-2-yl radical onto M2
Elemental_2E1P R# + M2 --> R-M2# kp02

# Reversible combination of the R-M1# radical adduct with SG1
Elemental_2E1P R-M1# + N# --> R-M1N kcR1
# Reversible combination of the R-M2# radical adduct with SG1
Elemental_2E1P R-M2# + N# --> R-M2N kcR2
# Homolytic dissociation of the SG1-terminated R-M1N alkoxyamines
Elemental_1E2P R-M1N --> R-M1# + N# kdR1
# Homolytic dissociation of the SG1-terminated R-M2N alkoxyamines
Elemental_1E2P R-M2N --> R-M2# + N# kdR2
# Homotermination between R-M1# radical adducts
Elemental_2E1P R-M1# + R-M1# --> R-M1-M1-R ktR1R1
# Termination between R-M1# and R-M2# radical adducts
Elemental_2E1P R-M1# + R-M2# --> R-M1-M2-R ktR1R2
# Homotermination between R-M2# radical adducts
Elemental_2E1P R-M2# + R-M2# --> R-M2-M2-R ktR2R2
# Addition of the R-M1# radical adduct onto M1
InitiationCopolymer11 R-M1# + M1 --> P-M1-M1# kpR11
# Addition of the R-M1# radical adduct onto M2
InitiationCopolymer12 R-M1# + M2 --> P-M1-M2# kpR12
# Addition of the R-M2# radical adduct onto M1
InitiationCopolymer21 R-M2# + M1 --> P-M2-M1# kpR21
# Addition of the R-M2# radical adduct onto M2
InitiationCopolymer22 R-M2# + M2 --> P-M2-M2# kpR22

# Additionb of the PM1M1# propagating macroradical onto M1
PropagationCopolymer1 P-M1-M1# + M1 --> P-M1-M1# kp111
# Additionb of the PM2M2# propagating macroradical onto M2
PropagationCopolymer2 P-M2-M2# + M2 --> P-M2-M2# kp222
# Additionb of the PM1M1# propagating macroradical onto M2
PropagationCopolymer2 P-M1-M1# + M2 --> P-M1-M2# kp112
# Additionb of the PM1M2# propagating macroradical onto M1
PropagationCopolymer1 P-M1-M2# + M1 --> P-M2-M1# kp121
# Additionb of the PM1M2# propagating macroradical onto M2
PropagationCopolymer2 P-M1-M2# + M2 --> P-M2-M2# kp122
# Additionb of the PM2M1# propagating macroradical onto M1
PropagationCopolymer1 P-M2-M1# + M1 --> P-M1-M1# kp211
# Additionb of the PM2M1# propagating macroradical onto M2
PropagationCopolymer2 P-M2-M1# + M2 --> P-M1-M2# kp212
# Additionb of the PM2M2# propagating macroradical onto M1
PropagationCopolymer1 P-M2-M2# + M1 --> P-M2-M1# kp221

# Reversible combination of the P-M1-M1# propagating macroradical with SG1
TransferCopolymer_PL-P P-M1-M1# + N# --> P-M1-M1N kc1
# Reversible combination of the P-M2-M1# propagating macroradical with SG1
TransferCopolymer_PL-P P-M2-M1# + N# --> P-M2-M1N kc1
# Reversible combination of the P-M2-M2# propagating macroradical with SG1
TransferCopolymer_PL-P P-M2-M2# + N# --> P-M2-M2N kc2
# Reversible combination of the P-M1-M2# propagating macroradical with SG1
TransferCopolymer_PL-P P-M1-M2# + N# --> P-M1-M2N kc2
# Homolytic dissociation of the SG1-terminated P-M1-M1N alkoxyamine
TransferCopolymer_P-PL P-M1-M1N --> P-M1-M1# + N# kd11
# Homolytic dissociation of the SG1-terminated P-M1-M2N alkoxyamine
TransferCopolymer_P-PL P-M1-M2N --> P-M1-M2# + N# kd12
# Homolytic dissociation of the SG1-terminated P-M2-M1N alkoxyamine
TransferCopolymer_P-PL P-M2-M1N --> P-M2-M1# + N# kd21
# Homolytic dissociation of the SG1-terminated P-M2-M2N alkoxyamine
TransferCopolymer_P-PL P-M2-M2N --> P-M2-M2# + N# kd22

# Irreversible termination between a P-M1-M1# and a P-M1-M1# propagating macroradical by disproportionation
TerminationDispCopolymer P-M1-M1# + P-M1-M1# --> D + D ktd11
# Irreversible termination between a P-M1-M1# and a P-M2-M1# propagating macroradical by disproportionation
TerminationDispCopolymer P-M1-M1# + P-M2-M1# --> D + D ktd11
# Irreversible termination between a P-M1-M1# and a P-M2-M1# propagating macroradical by disproportionation
TerminationDispCopolymer P-M2-M1# + P-M2-M1# --> D + D ktd11
# Irreversible termination between a P-M1-M1# and a P-M2-M2# propagating macroradical by disproportionation
TerminationDispCopolymer P-M1-M1# + P-M2-M2# --> D + D ktd12
# Irreversible termination between a P-M1-M1# and a P-M1-M2# propagating macroradical by disproportionation
TerminationDispCopolymer P-M1-M1# + P-M1-M2# --> D + D ktd12
# Irreversible termination between a P-M2-M1# and a P-M1-M2# propagating macroradical by disproportionation
TerminationDispCopolymer P-M2-M1# + P-M1-M2# --> D + D ktd12
# Irreversible termination between a P-M2-M2# and a P-M2-M2# propagating macroradical by disproportionation
TerminationDispCopolymer P-M2-M2# + P-M2-M2# --> D + D ktd22
# Irreversible termination between a P-M2-M2# and a P-M1-M2# propagating macroradical by disproportionation
TerminationDispCopolymer P-M2-M2# + P-M1-M2# --> D + D ktd22
# Irreversible termination between a P-M1-M2# and a P-M1-M2# propagating macroradical by disproportionation
TerminationDispCopolymer P-M1-M2# + P-M1-M2# --> D + D ktd22

# Irreversible termination between a P-M1-M1# and a P-M1-M1# propagating macroradical by combination
TerminationCombCopolymer P-M1-M1# + P-M1-M1# --> D ktc11
# Irreversible termination between a P-M1-M1# and a P-M2-M1# propagating macroradical by combination
TerminationCombCopolymer P-M1-M1# + P-M2-M1# --> D ktc11
# Irreversible termination between a P-M1-M1# and a P-M2-M1# propagating macroradical by combination
TerminationCombCopolymer P-M2-M1# + P-M2-M1# --> D ktc11
# Irreversible termination between a P-M1-M1# and a P-M2-M2# propagating macroradical by combination
TerminationCombCopolymer P-M1-M1# + P-M2-M2# --> D ktc12
# Irreversible termination between a P-M1-M1# and a P-M1-M2# propagating macroradical by combination
TerminationCombCopolymer P-M1-M1# + P-M1-M2# --> D ktc12
# Irreversible termination between a P-M2-M1# and a P-M1-M2# propagating macroradical by combination
TerminationCombCopolymer P-M2-M1# + P-M1-M2# --> D ktc12
# Irreversible termination between a P-M2-M2# and a P-M2-M2# propagating macroradical by combination
TerminationCombCopolymer P-M2-M2# + P-M2-M2# --> D ktc22
# Irreversible termination between a P-M2-M2# and a P-M1-M2# propagating macroradical by combination
TerminationCombCopolymer P-M2-M2# + P-M1-M2# --> D ktc22
# Irreversible termination between a P-M1-M2# and a P-M1-M2# propagating macroradical by combination
TerminationCombCopolymer P-M1-M2# + P-M1-M2# --> D ktc22

# -hydrogen transfer from a P-M1-M1# propagating macroradicals to the nitroxide SG1
TransferCopolymer_PL-P P-M1-M1# + N# --> DbHtr kbHtr
# -hydrogen transfer from a P-M1-M1# propagating macroradicals to the nitroxide SG1
TransferCopolymer_PL-P P-M2-M1# + N# --> DbHtr kbHtr

puts "call listReactions"
# Interface function
ListAllReactions

# set reaction temperature 
Temperature 90.0

# set frequency for writing intermediate results 
Setdt 60.0

SetCopolymerization on

# define number of molecules for simulation 1e8..1e10
InitSimulation 1e9

# start simulation to overall reaction time
Simulation 3600.0
