// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2012 Marco Drache, Georg Drache
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

/*
 * mcPolymer.h
 *
 *  Created on: 24.08.2008
 *      Author: drache
 */

#ifndef MCPOLYMER_H_
#define MCPOLYMER_H_
#include "chain.h"
#include "polymer.h"
#include <vector>

using namespace std;

class Reactant {
public:
	Reactant(int _id, double _molcount, bool _macro, int _type,
			double _addvalue);

	~Reactant();
	int getID() {
		return id;
	}
	double getMolcount() {
		return molcount;
	}
	polymerSpecies* getPolymerS() {
		if (macro) {
			return poly;
		} else {
			return NULL;
		}
	}
	bool isMacro() {
		if (macro) {
			return true;
		} else {
			return false;
		}
	}
	bool isMacroD() {
		if (macroD) {
			return true;
		} else {
			return false;
		}
	}
	inline int getType() {
		return type;
	}
	inline double getAddvalue() {
		return addvalue;
	}
private:
	int id;
	int type;
	double addvalue;
	double molcount;
	bool macro;
	bool macroD;
	bool macroH;
	polymerSpecies* poly;
};

class MCRateC {
public:
	MCRateC(int _id, double _value);
	~MCRateC();
	int getID() {
		return id;
	}
	double getValue() {
		return value;
	}
	void setValue(double _value) {
		value = _value;
	}
private:
	int id;
	double value;
};

class Reaction {
public:
	Reaction(int _id, MCRateC* _k, int _type);
	~Reaction();
	void addEduct(Reactant *rp);
	void addProduct(Reactant *pp);
	int getType() {
		return type;
	}
	int getID() {
		return id;
	}
	vector<Reactant*> educt;
	vector<Reactant*> product;
	MCRateC* k;
private:
	int id;
	int type;

};
class Simulation;
class CoreInterface {
public:
	CoreInterface() {
	}
	;
	~CoreInterface() {
	}
	;
	int getID() {
		return id;
	}
	bool addReactant(int _id, double _molcount, bool _macro, int _type,
			double _addvalue);
	bool addReaction(int _id, int _idMCRateC, int _type);
	bool addMCRateC(int _id, double _value);
	bool setMCRateC(int _id, double _value);
	bool addEduct2Reaction(int _idReaction, int _idReactant);
	bool addProduct2Reaction(int _idReaction, int _idReactant);
	int countReactants();
	int countMCRateCs();
	int countReactions();
	int initSimulation();
	void updateMCRates();
	int runSimulation(int _cmdCount, double _destTime);
	vector<Reactant*> Reactants;
	vector<MCRateC*> MCRateCs;
	vector<Reaction*> Reactions;
	bool modeMacro;
	bool modeCopo;
	bool modeHomopolymerization;
private:
	int id;

	Simulation *sim;
};
class Simulation {
public:
	Simulation(CoreInterface *i);
	~Simulation() {
	}
	;
	int runTo(int _cmdCount, double destTime, bool modeM, bool modeCP,
			bool modeHP);
	void updateMCRates(CoreInterface *i);
private:
	struct FRkt {
		double k;
		int cEducts;
		double **educts;
		polymerSpecies **polyeducts;
		int cProducts;
		double **products;
		polymerSpecies **polyproducts;
		int type;
		bool self;
		double addvalue;
		bool *rktUpdate;
	};
	double *reactants;
	int cReactants;

	FRkt *rkt;
	bool *rktUpdate;
	double *pr;
	double *cPr;
	double t, dt;
	int rktSize;
	CoreInterface *ci;

};
#endif
