# mcPolymer - Monte Carlo simulation program of radical polymerization
# Copyright (C) 2008 - 2012 Marco Drache, Georg Drache
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# __________________________________________________________________
# 0. global settings
set tcl_precision 10
set nSpecies 0
set nReactions 0
set nRateConstants 0
set simulationControl(hamielec) 0
set simulationControl(MA_Model) 0
set simulationControl(modemacro) 0
set simulationControl(modehomopolymerization) 0
set simulationControl(modecopo) 0
set simulationControl(cleanupresults) 1

# __________________________________________________________________
# 1. Species definition

# find species and doubly definitions
proc checkDefinedSpecies { sname } {
	global nSpecies
	global Species
	for {set i 1} {$i<=$nSpecies} {incr i} {
		if {$Species($i,name)==$sname} {
			return $i
		}
	}
	return 0
}

# 1.1 low molecular species

proc Species { sname } {
	global nSpecies
	global Species
	if {[checkDefinedSpecies $sname]!=0} {
		puts "Error - Species - doubly definition:\tSpecies $sname"
		exit 1
	}
	incr nSpecies
	set Species($nSpecies,name) $sname
	set Species($nSpecies,type) "Species"
	set Species($nSpecies,macromolecular) 0
	set Species($nSpecies,concentration) 0.0
}

# Monomer Name
proc Monomer { sname } {
	global nSpecies
	global Species
	if {[checkDefinedSpecies $sname]!=0} {
		puts "Error - Species - doubly definition:\tMonomer $sname"
		exit 1
	}
	incr nSpecies
	set Species($nSpecies,name) $sname
	set Species($nSpecies,type) "Monomer"
	set Species($nSpecies,macromolecular) 0
	set Species($nSpecies,concentration) 0.0
}

# Initiator Name f
proc Initiator { sname sf } {
	global nSpecies
	global Species
	if {[checkDefinedSpecies $sname]!=0} {
		puts "Error - Species - doubly definition:\tInitiator $sname"
		exit 1
	}
	incr nSpecies
	set Species($nSpecies,name) $sname
	set Species($nSpecies,type) "Initiator"
	set Species($nSpecies,f) $sf
	set Species($nSpecies,macromolecular) 0
	set Species($nSpecies,concentration) 0.0
}

# 1.2 macromolecular species

# SpeciesMacro Name
proc SpeciesMacro { sname } {
	global nSpecies
	global Species
	if {[checkDefinedSpecies $sname]!=0} {
		puts "Error - Species - doubly definition:\tSpeciesMacro $sname"
		exit 1
	}
	incr nSpecies
	set Species($nSpecies,name) $sname
	set Species($nSpecies,type) "Macro"
	set Species($nSpecies,macromolecular) 1
	set Species($nSpecies,concentration) 0.0
}

# D - dead polymer
# SpeciesMacroD Name
proc SpeciesMacroD { sname } {
	global nSpecies
	global Species
	if {[checkDefinedSpecies $sname]!=0} {
		puts "Error - Species - doubly definition:\tSpeciesMacroD $sname"
		exit 1
	}
	incr nSpecies
	set Species($nSpecies,name) $sname
	set Species($nSpecies,type) "MacroD"
	set Species($nSpecies,macromolecular) 2
	set Species($nSpecies,concentration) 0.0
}

# SpeciesMacroHomopolymer Name
proc SpeciesMacroHomopolymer { sname } {
	global nSpecies
	global Species
	if {[checkDefinedSpecies $sname]!=0} {
		puts "Error - Species - doubly definition:\SpeciesMacroHomopolymer $sname"
		exit 1
	}
	incr nSpecies
	set Species($nSpecies,name) $sname
	set Species($nSpecies,type) "Macro"
	set Species($nSpecies,macromolecular) 3
	set Species($nSpecies,concentration) 0.0
}
# SpeciesMacroCopolymer Name
proc SpeciesMacroCopolymer { sname } {
	global nSpecies
	global Species
	if {[checkDefinedSpecies $sname]!=0} {
		puts "Error - Species - doubly definition:\tSpeciesMacroCopolymer $sname"
		exit 1
	}
	incr nSpecies
	set Species($nSpecies,name) $sname
	set Species($nSpecies,type) "MacroCopolymer"
	set Species($nSpecies,macromolecular) 1
	set Species($nSpecies,concentration) 0.0
}

# SpeciesMacroCopolymerD Name
proc SpeciesMacroCopolymerD { sname } {
	global nSpecies
	global Species
	if {[checkDefinedSpecies $sname]!=0} {
		puts "Error - Species - doubly definition:\tSpeciesMacroCopolymerD $sname"
		exit 1
	}
	incr nSpecies
	set Species($nSpecies,name) $sname
	set Species($nSpecies,type) "MacroCopolymerD"
	set Species($nSpecies,macromolecular) 1
	set Species($nSpecies,concentration) 0.0
}


# special - linear polymer, 2 chains
# SpeciesMacro_P-O-P Name
proc SpeciesMacro_P-O-P { sname } {
	global nSpecies
	global Species
	if {[checkDefinedSpecies $sname]!=0} {
		puts "Error - Species - doubly definition:\tMacro_P-O-P $sname"
		exit 1
	}
	incr nSpecies
	set Species($nSpecies,name) $sname
	set Species($nSpecies,type) "Macro_P-O-P"
	set Species($nSpecies,macromolecular) 3
	set Species($nSpecies,concentration) 0.0
}

# special - star polymer, 3 chains
# SpeciesMacro_P-O<PP Name
proc SpeciesMacro_P-O<PP { sname } {
	global nSpecies
	global Species
	if {[checkDefinedSpecies $sname]!=0} {
		puts "Error - Species - doubly definition:\tMacro_P-O<PP $sname"
		exit 1
	}
	incr nSpecies
	set Species($nSpecies,name) $sname
	set Species($nSpecies,type) "Macro_P-O<PP"
	set Species($nSpecies,macromolecular) 4
	set Species($nSpecies,concentration) 0.0
}


# __________________________________________________________________
# 2. set concentration of defined species / mol/L

# Concentration SpeciesName Concentration
proc Concentration { sname sc} {
	global nSpecies
	global Species
	set sindex [checkDefinedSpecies $sname]
	if {$sindex!=0} {
		set Species($sindex,concentration) $sc
		return
	}
	puts "Error - Concentration - species not found:\t$sname"
	exit
}

# debug function
# ListAllSpecies
proc ListAllSpecies { } {
	global nSpecies
	global Species	
	puts "$nSpecies species defined:"
	puts "Name\tType\tConcentration / mol/L"
	for {set i 1} {$i<=$nSpecies} {incr i} {
		puts [format "$Species($i,name)\t$Species($i,type)\t$Species($i,concentration)"]
	}
	puts ""
}

# __________________________________________________________________
# 3. reaction rate constants definition

# find reaction rates and doubly definitions
proc checkDefinedRateConstant { cname } {
	global nRateConstants
	global Constant
	for {set i 1} {$i<=$nRateConstants} {incr i} {
		if {$Constant($i,name)==$cname} {
			return $i
		}
	}
	return 0
}

# temperature independent
# RateConstant RateConstantName Value (s,L,mol)
proc RateConstant { cname cvalue} {
	global nRateConstants
	global Constant
	if {[checkDefinedRateConstant $cname]!=0} {
		puts "Error - RateConstant - doubly definition:\t$cname"
		exit
	}
	incr nRateConstants
	set Constant($nRateConstants,name) $cname
	set Constant($nRateConstants,type) "tempIndep"
	set Constant($nRateConstants,value) $cvalue
	set Constant($nRateConstants,k0) 0.0
	set Constant($nRateConstants,ea) 0.0
}

# temperature dependent
# RateConstantArrhenius RateConstantName k0 (s,L,mol) Ea/R (K)
proc RateConstantArrhenius { cname ck0 cea} {
	global nRateConstants
	global Constant
	if {[checkDefinedRateConstant $cname]!=0} {
		puts "Error - RateConstantArrhenius - doubly definition:\t$cname"
		exit
	}
	incr nRateConstants
	set Constant($nRateConstants,name) $cname
	set Constant($nRateConstants,type) "tempDep"
	set Constant($nRateConstants,k0) $ck0
	set Constant($nRateConstants,ea) $cea
	set Constant($nRateConstants,value) 0.0
}

# debug function
# ListAllRateConstants
proc ListAllRateConstants { } {
	global nRateConstants
	global Constant	
	puts "$nRateConstants rate constants defined:"
	puts "Name\tValue\t\[Value\]"
	for {set i 1} {$i<=$nRateConstants} {incr i} {
		puts -nonewline [format "$Constant($i,name)\t"]
		if {$Constant($i,type)=="tempIndep"} {
			puts [format "$Constant($i,value)"]
		} else {
			puts [format "$Constant($i,k0)\t$Constant($i,ea)"]
		}
	}
	puts ""
}

# __________________________________________________________________
# 4. Reaction definition

# find species and check species type
proc checkDefinedSpeciesType {sourcename sname stype} {
	global nSpecies
	global Species
	set sindex [checkDefinedSpecies $sname]
	if {$sindex==0} {
		puts "Error - reaction definition $sourcename - undefined species:\t$sname"
		exit
	}
	if {$stype=="anySpecies"} {		
		return 1
	}
	if {$stype=="MacroExact"} {
		if {($Species($sindex,type)!="Macro") && ($Species($sindex,type)!="MacroD")} {			
			puts "Error - $sourcename: $Species($sindex,name) $Species($sindex,type) is not type SpeciesMacro or SpeciesMacroD"
			exit
		} else {
			return 1
		}
	}
	if {$stype=="Macro"} {
		if {[string compare "Macro" $Species($sindex,type)]>0} {
			puts "Error - $sourcename: $Species($sindex,name) is not type SpeciesMacro*"
			exit
		}
	} else {
		if {$stype=="Macro_P-O-P"} {
			if {[string compare "Macro_P-O-P" $Species($sindex,type)]>0} {
				puts "Error - $sourcename: $Species($sindex,name) is not type Macro_P-O-P"
				exit
			}
		} else {
			if {$stype=="Macro_P-O<PP"} {
				if {[string compare "Macro_P-O<PP" $Species($sindex,type)]>0} {
					puts "Error - $sourcename: $Species($sindex,name) is not type Macro_P-O<PP"
					exit
				}
			} else {
				if {$Species($sindex,type)!=$stype} {			
					puts "Error - $sourcename: $Species($sindex,name) is not type $stype"
					exit
				}
			}
		}
	}
	return 1
}

# debug function
# ListAllReactions
proc ListAllReactions { } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	
	puts "$nReactions reactions defined:"
	puts "Name\tEductlist\t-->\tProductlist\trate constant"
	for {set i 1} {$i<=$nReactions} {incr i} {
		puts -nonewline [format "$Reaction($i,type) "]
		for {set j 1} {$j<=$Reaction($i,nEducts)} {incr j} {
			if {$j>1} {puts -nonewline "+ "}
			puts -nonewline [format "$Reaction($i,educt$j) "]
		}
		puts -nonewline "--> "
		for {set j 1} {$j<=$Reaction($i,nProducts)} {incr j} {
			if {$j>1} {puts -nonewline "+ "}
			puts -nonewline [format "$Reaction($i,product$j) "]
		}
		puts [format "$Reaction($i,constant)"]
	}
	puts ""
}

# 4.1 elemental reactions

# elemental reaction: 1 educt, 1 product
# Elemental_1E1P SpeciesName1 --> SpeciesName2 RateConstantName
proc Elemental_1E1P { anyA arrow anyB rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Elemental_1E1P $anyA anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_1E1P $anyB anySpecies]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Elemental_1E1P - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition Elemental_1E1P - \"-->\" missing"
		exit
	}
	incr nReactions
	set Reaction($nReactions,educt1) $anyA 
	set Reaction($nReactions,nEducts) 1
	set Reaction($nReactions,product1) $anyB 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "Elemental_1E1P"	
}

# elemental reaction: 2 educts, 1 product
# Elemental_2E1P SpeciesName1 + SpeciesName2 --> SpeciesName3 RateConstantName
proc Elemental_2E1P { anyA plus anyB arrow anyC rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Elemental_2E1P $anyA anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_2E1P $anyB anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_2E1P $anyC anySpecies]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Elemental_2E1P - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition Elemental_2E1P - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts  "Error - reaction definition Elemental_2E1P - \"-->\" missing"
		exit
	}
	incr nReactions
	set Reaction($nReactions,educt1) $anyA 
	set Reaction($nReactions,educt2) $anyB 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $anyC 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "Elemental_2E1P"	
}

# elemental reaction: 1 educt, 2 products
# Elemental_1E2P SpeciesName1 --> SpeciesName2 + SpeciesName3 RateConstantName
proc Elemental_1E2P { anyA arrow anyB plus anyC rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Elemental_1E2P $anyA anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_1E2P $anyB anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_1E2P $anyC anySpecies]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Elemental_1E2P - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition Elemental_1E2P - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts  "Error - reaction definition Elemental_1E2P - \"-->\" missing"
		exit
	}
	
	incr nReactions
	set Reaction($nReactions,educt1) $anyA 
	set Reaction($nReactions,nEducts) 1
	set Reaction($nReactions,product1) $anyB 
	set Reaction($nReactions,product2) $anyC 
	set Reaction($nReactions,nProducts) 2	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "Elemental_1E2P"	
}

# elemental reaction: 2 educts, 2 products
# Elemental_2E2P SpeciesName1 + SpeciesName2 --> SpeciesName3 + SpeciesName4 RateConstantName
proc Elemental_2E2P { anyA plus1 anyB arrow anyC plus2 anyD  rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Elemental_2E2P $anyA anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_2E2P $anyB anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_2E2P $anyC anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_2E2P $anyD anySpecies]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Elemental_2E2P - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus1 != "+"} {
		puts "Error - reaction definition Elemental_2E2P - \"+\" missing"
		exit
	}
	if {$plus2 != "+"} {
		puts "Error - reaction definition Elemental_2E2P - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts  "Error - reaction definition Elemental_2E2P - \"-->\" missing"
		exit
	}
	incr nReactions
	set Reaction($nReactions,educt1) $anyA 
	set Reaction($nReactions,educt2) $anyB 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $anyC 
	set Reaction($nReactions,product2) $anyD 
	set Reaction($nReactions,nProducts) 2	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "Elemental_2E2P"	
}

# elemental reaction: 2 educts, 2 products
# Elemental_2E2P SpeciesName1 + SpeciesName2 --> SpeciesName3 + SpeciesName4 + SpeciesName5 RateConstantName
proc Elemental_2E3P { anyA plus1 anyB arrow anyC plus2 anyD plus3 anyE rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Elemental_2E3P $anyA anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_2E3P $anyB anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_2E3P $anyC anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_2E3P $anyC anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_2E3P $anyC anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Elemental_2E3P $anyC anySpecies]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Elemental_2E3P - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus1 != "+"} {
		puts "Error - reaction definition Elemental_2E3P - \"+\" missing"
		exit
	}
	if {$plus2 != "+"} {
		puts "Error - reaction definition Elemental_2E3P - \"+\" missing"
		exit
	}
	if {$plus3 != "+"} {
		puts "Error - reaction definition Elemental_2E3P - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts  "Error - reaction definition Elemental_2E3P - \"-->\" missing"
		exit
	}
	
	incr nReactions
	set Reaction($nReactions,educt1) $anyA 
	set Reaction($nReactions,educt2) $anyB 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $anyC 
	set Reaction($nReactions,product2) $anyD 
	set Reaction($nReactions,product3) $anyE 
	set Reaction($nReactions,nProducts) 3	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "Elemental_2E3P"	
}

# initiator decomposition
# InitiatorDecomposition InitiatorName  --> SpeciesName1 + SpeciesName2 RateConstantName
proc InitiatorDecomposition { rinitiator arrow rradical1 plus rradical2 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType InitiatorDecomposition $rinitiator Initiator]==0} {
		exit
	}
	if {[checkDefinedSpeciesType InitiatorDecomposition $rradical1 anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType InitiatorDecomposition $rradical2 anySpecies]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition InitiatorDecomposition - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition InitiatorDecomposition - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts  "Error - reaction definition InitiatorDecomposition - \"-->\" missing"
		exit
	}
	
	incr nReactions
	set Reaction($nReactions,educt1) $rinitiator 
	set Reaction($nReactions,nEducts) 1
	set Reaction($nReactions,product1) $rradical1 
	set Reaction($nReactions,product2) $rradical1 
	set Reaction($nReactions,nProducts) 2	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "InitiatorDecomposition"	
}

# 4.2 macromolecular reactions

# ThermalInitiation
# ThermalInitiation MonomerName + MonomerName + MonomerName  --> SpeciesMacroName + SpeciesMacroName RateConstantName
proc ThermalInitiation { m1 plus1 m2 plus2 m3 arrow p1 plus3 p2 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType ThermalInitiation $m1 Monomer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType ThermalInitiation $p1 Macro]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition ThermalInitiation - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus1 != "+"} {
		puts "Error - reaction definition ThermalInitiation - \"+\" missing"
		exit
	}
	if {$plus2 != "+"} {
		puts "Error - reaction definition ThermalInitiation - \"+\" missing"
		exit
	}
	if {$plus3 != "+"} {
		puts "Error - reaction definition ThermalInitiation - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition ThermalInitiation - \"-->\" missing"
		exit
	}
	incr nReactions
	set Reaction($nReactions,educt1) $m1 
	set Reaction($nReactions,educt2) $m1
	set Reaction($nReactions,educt3) $m1
	set Reaction($nReactions,nEducts) 3
	set Reaction($nReactions,product1) $p1 
	set Reaction($nReactions,product2) $p2 
	set Reaction($nReactions,nProducts) 2	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "ThermalInitiation"	
}

# Propagation
# Propagation SpeciesMacroName + MonomerName --> SpeciesMacroName RateConstantName
proc Propagation { p1 plus m arrow p11 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Propagation $p1 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Propagation $m Monomer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Propagation $p11 Macro]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Propagation - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition Propagation - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition Propagation - \"-->\" missing"
		exit
	}
	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $m 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p11 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "Propagation"	
}

# Termination
# Termination SpeciesMacroName1 + SpeciesMacroName1 --> SpeciesMacroDName2 RateConstantName
proc Termination { p1 plus p2 arrow p12 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Termination $p1 MacroExact]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Termination $p2 MacroExact]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Termination $p12 MacroD]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Termination - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition Termination - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition Termination - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $p2 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p12 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant
	if {$p1 == $p2} {
		set Reaction($nReactions,type) "Termination2P"	
	} else {
		set Reaction($nReactions,type) "TerminationD"
	}
}

# Transfer2monomer
# Transfer2monomer SpeciesMacroName1 + MonomerName --> SpeciesMacroDName1 + SpeciesMacroName1 RateConstantName
proc Transfer2monomer { p1 plus m arrow p1D plus1 p2 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Transfer2monomer $p1 MacroExact]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Transfer2monomer $m Monomer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Transfer2monomer $p1D MacroD]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Transfer2monomer $p2 MacroExact]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Transfer2monomer - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition Transfer2monomer - \"+\" missing"
		exit
	}
	if {$plus1 != "+"} {
		puts "Error - reaction definition Transfer2monomer - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition Transfer2monomer - \"-->\" missing"
		exit
	}


	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $m 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p1D 
	set Reaction($nReactions,product2) $p2 
	set Reaction($nReactions,nProducts) 2	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "Transfer2monomer"	
}

# Initiation
# Initiation SpeciesName1 + SpeciesName2 --> SpeciesMacroName RateConstantName
proc Initiation { r plus m arrow p1 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Initiation $r anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Initiation $m anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Initiation $p1 MacroExact]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Initiation - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition Initiation - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition Initiation - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $r 
	set Reaction($nReactions,educt2) $m 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p1 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "Initiation"	
}

# Transfer_PL-P
# Transfer_PL-P SpeciesMacroName1 + SpeciesName --> SpeciesMacroName2 RateConstantName
proc Transfer_PL-P { p plus a arrow pa rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Transfer_PL-P $p Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Transfer_PL-P $a anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Transfer_PL-P $pa Macro]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Transfer_PL-P - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition Transfer_PL-P - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition Transfer_PL-P - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $p 
	set Reaction($nReactions,educt2) $a 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $pa 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "Transfer_PL-P"	
}

# Transfer_P-PL
# Transfer_P-PL SpeciesMacroName1 --> SpeciesMacroName2 + SpeciesName RateConstantName
proc Transfer_P-PL { pa arrow p plus a rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Transfer_P-PL $pa Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Transfer_P-PL $p Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Transfer_P-PL $p anySpecies]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Transfer_P-PL - undefined reaction rate:\t$rconstant"
		exit
	}
	
	if {$plus != "+"} {
		puts "Error - reaction definition Transfer_P-PL - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition Transfer_P-PL - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $pa 
	set Reaction($nReactions,nEducts) 1
	set Reaction($nReactions,product1) $p 
	set Reaction($nReactions,product2) $a 
	set Reaction($nReactions,nProducts) 2	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "Transfer_P-PL"	
}

# Transfer_PL-PL
# Transfer_PL-PL SpeciesMacroName1 + SpeciesName1 --> SpeciesMacroName2 + SpeciesName2 RateConstantName
proc Transfer_PL-PL { p1 plus1 a arrow p2 plus2 b rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType Transfer_PL-PL $p1 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Transfer_PL-PL $a anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Transfer_PL-PL $p2 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType Transfer_PL-PL $b anySpecies]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition Transfer_PL-PL - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus1 != "+"} {
		puts "Error - reaction definition Transfer_PL-PL - \"+\" missing"
		exit
	}
	if {$plus2 != "+"} {
		puts "Error - reaction definition Transfer_PL-PL - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition Transfer_PL-PL - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $a 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p2 
	set Reaction($nReactions,product2) $b 
	set Reaction($nReactions,nProducts) 2	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "Transfer_PL-PL"	
}

# ChainAddition
# ChainAddition SpeciesMacroName1 + SpeciesMacro(nChains)Name2 --> SpeciesMacro(n+1Chains)Name3 RateConstantName
proc ChainAddition { p1 plus p2 arrow p3 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType ChainAddition $p1 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType ChainAddition $p2 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType ChainAddition $p3 Macro]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition ChainAddition - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition ChainAddition - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition ChainAddition - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $p2
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p3 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "ChainAddition"	
}

# ChainFragmentation
# ChainFragmentation SpeciesMacro(nChains)Name1 --> SpeciesMacro(n-1Chains)Name2 + SpeciesMacroName3 RateConstantName
proc ChainFragmentation { p1 arrow p2 plus p3 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType ChainFragmentation $p1 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType ChainFragmentation $p2 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType ChainFragmentation $p3 Macro]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition ChainFragmentation - undefined reaction rate:\t$rconstant"
		exit
	}
	
	if {$plus != "+"} {
		puts "Error - reaction definition ChainFragmentation - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition ChainFragmentation - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,nEducts) 1
	set Reaction($nReactions,product1) $p2 
	set Reaction($nReactions,product2) $p3
	set Reaction($nReactions,nProducts) 2	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "ChainFragmentation"	
}

# copolymerization

# InitiationCopolymer11
# InitiationCopolymer11 SpeciesName1 + SpeciesName2 --> SpeciesMacroCopolymerName3 RateConstantName
proc InitiationCopolymer11 { p1 plus p2 arrow p3 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType InitiationCopolymer11 $p1 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType InitiationCopolymer11 $p2 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType InitiationCopolymer11 $p3 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition InitiationCopolymer11 - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition InitiationCopolymer11 - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition InitiationCopolymer11 - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $p2
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p3 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "InitiationCopolymer11"	
}

# InitiationCopolymer12
# InitiationCopolymer12 SpeciesName1 + SpeciesName2 --> SpeciesMacroCopolymerName3 RateConstantName
proc InitiationCopolymer12 { p1 plus p2 arrow p3 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType InitiationCopolymer12 $p1 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType InitiationCopolymer12 $p2 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType InitiationCopolymer12 $p3 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition InitiationCopolymer12 - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition InitiationCopolymer12 - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition InitiationCopolymer12 - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $p2
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p3 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "InitiationCopolymer12"	
}

# InitiationCopolymer21
# InitiationCopolymer21 SpeciesName1 + SpeciesName2 --> SpeciesMacroCopolymerName3 RateConstantName
proc InitiationCopolymer21 { p1 plus p2 arrow p3 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType InitiationCopolymer21 $p1 anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType InitiationCopolymer21 $p2 Monomer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType InitiationCopolymer21 $p3 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition InitiationCopolymer21 - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition InitiationCopolymer21 - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition InitiationCopolymer21 - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $p2
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p3 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "InitiationCopolymer21"	
}

# InitiationCopolymer22
# InitiationCopolymer22 SpeciesName1 + SpeciesName2 --> SpeciesMacroCopolymerName3 RateConstantName
proc InitiationCopolymer22 { p1 plus p2 arrow p3 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType InitiationCopolymer22 $p1 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType InitiationCopolymer22 $p2 Macro]==0} {
		exit
	}
	if {[checkDefinedSpeciesType InitiationCopolymer22 $p3 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition InitiationCopolymer22 - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition InitiationCopolymer22 - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition InitiationCopolymer22 - \"-->\" missing"
		exit
	}
	
	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $p2
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p3 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "InitiationCopolymer22"	
}

# PropagationCopolymer1
# PropagationCopolymer1 SpeciesMacroCopolymerName + MonomerName --> SpeciesMacroCopolymerName RateConstantName
proc PropagationCopolymer1 { p1 plus m arrow p11 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType PropagationCopolymer1 $p1 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType PropagationCopolymer1 $m Monomer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType PropagationCopolymer1 $p11 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition PropagationCopolymer1 - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition PropagationCopolymer1 - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition PropagationCopolymer1 - \"-->\" missing"
		exit
	}
	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $m 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p11 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "PropagationCopolymer1"	
}

# PropagationCopolymer2
# PropagationCopolymer2 SpeciesMacroCopolymerName + MonomerName --> SpeciesMacroCopolymerName RateConstantName
proc PropagationCopolymer2 { p1 plus m arrow p11 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType PropagationCopolymer2 $p1 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType PropagationCopolymer2 $m Monomer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType PropagationCopolymer2 $p11 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition PropagationCopolymer2 - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition PropagationCopolymer2 - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition PropagationCopolymer2 - \"-->\" missing"
		exit
	}
	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $m 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p11 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "PropagationCopolymer2"	
}

# TransferCopolymer_PL-P
# TransferCopolymer_PL-P SpeciesMacroCopolymerName1 + SpeciesName --> SpeciesMacroCopolymerName2 RateConstantName
proc TransferCopolymer_PL-P { p plus a arrow pa rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType TransferCopolymer_PL-P $p MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType TransferCopolymer_PL-P $a anySpecies]==0} {
		exit
	}
	if {[checkDefinedSpeciesType TransferCopolymer_PL-P $pa MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition TransferCopolymer_PL-P - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition TransferCopolymer_PL-P - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition TransferCopolymer_PL-P - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $p 
	set Reaction($nReactions,educt2) $a 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $pa 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "TransferCopolymer_PL-P"	
}

# TransferCopolymer_P-PL
# TransferCopolymer_P-PL SpeciesMacroCopolymerName1 --> SpeciesMacroCopolymerName2 + SpeciesName RateConstantName
proc TransferCopolymer_P-PL { pa arrow p plus a rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType TransferCopolymer_P-PL $pa MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType TransferCopolymer_P-PL $p MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType TransferCopolymer_P-PL $p anySpecies]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition TransferCopolymer_P-PL - undefined reaction rate:\t$rconstant"
		exit
	}
	
	if {$plus != "+"} {
		puts "Error - reaction definition TransferCopolymer_P-PL - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition TransferCopolymer_P-PL - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $pa 
	set Reaction($nReactions,nEducts) 1
	set Reaction($nReactions,product1) $p 
	set Reaction($nReactions,product2) $a 
	set Reaction($nReactions,nProducts) 2	 
	set Reaction($nReactions,constant) $rconstant	
	set Reaction($nReactions,type) "TransferCopolymer_P-PL"	
}

# TerminationDispCopolymer
# TerminationDispCopolymer SpeciesMacroCopolymerName1 + SpeciesMacroCopolymerName2 --> SpeciesMacroCopolymerDName3 + SpeciesMacroCopolymerDName4 RateConstantName
proc TerminationDispCopolymer { p1 plus p2 arrow p3 plus1 p4 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType TerminationDispCopolymer $p1 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType TerminationDispCopolymer $p2 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType TerminationDispCopolymer $p3 MacroCopolymerD]==0} {
		exit
	}
	if {[checkDefinedSpeciesType TerminationDispCopolymer $p4 MacroCopolymerD]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition TerminationDispCopolymer - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition TerminationDispCopolymer - \"+\" missing"
		exit
	}
	if {$plus1 != "+"} {
		puts "Error - reaction definition TerminationDispCopolymer - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition TerminationDispCopolymer - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $p2 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p3 
	set Reaction($nReactions,product2) $p4 
	set Reaction($nReactions,nProducts) 2	 
	set Reaction($nReactions,constant) $rconstant
	set Reaction($nReactions,type) "TerminationDispCopolymer"
	
}

# TerminationCombCopolymer
# TerminationCombCopolymer SpeciesMacroCopolymerName1 + SpeciesMacroCopolymerName2 --> SpeciesMacroCopolymerDName3 RateConstantName
proc TerminationCombCopolymer { p1 plus p2 arrow p12 rconstant } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	

	if {[checkDefinedSpeciesType TerminationCombCopolymer $p1 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType TerminationCombCopolymer $p2 MacroCopolymer]==0} {
		exit
	}
	if {[checkDefinedSpeciesType TerminationCombCopolymer $p12 MacroCopolymerD]==0} {
		exit
	}
	if {[checkDefinedRateConstant $rconstant]==0} {
		puts "Error - reaction definition TerminationCombCopolymer - undefined reaction rate:\t$rconstant"
		exit
	}
	if {$plus != "+"} {
		puts "Error - reaction definition TerminationCombCopolymer - \"+\" missing"
		exit
	}
	if {$arrow != "-->"} {
		puts "Error - reaction definition TerminationCombCopolymer - \"-->\" missing"
		exit
	}

	incr nReactions
	set Reaction($nReactions,educt1) $p1 
	set Reaction($nReactions,educt2) $p2 
	set Reaction($nReactions,nEducts) 2
	set Reaction($nReactions,product1) $p12 
	set Reaction($nReactions,nProducts) 1	 
	set Reaction($nReactions,constant) $rconstant
	set Reaction($nReactions,type) "TerminationCombCopolymer"
	
}

# __________________________________________________________________
# 5. Simulation  settings

# Temperature
# Temperature temperature (C)   
proc Temperature { ttemp } {
	global Temperature
	set Temperature $ttemp
}

# Setdt
# Setdt TimeInterval (s)
proc Setdt { sdt } {
	global simulationControl
	set simulationControl(dt) $sdt	
}

# CleanupResults
# CleanupResults on / off
proc CleanupResults { cr } {
	global simulationControl
	if {$cr=="off"} {
		set simulationControl(cleanupresults) 0
	} else {
		set simulationControl(cleanupresults) 1
	} 
}

#SetCalculateMacromolecules
#SetCalculateMacromolecules on / off
proc SetCalculateMacromolecules { smacro } {
	global simulationControl
	if {$smacro=="off"} {
		set simulationControl(modemacro) 0
	} else {
		set simulationControl(modemacro) 1
	} 
}

#SetCalculateHomopolymers
#SetCalculateHomopolymers on / off
proc SetCalculateHomopolymers { smacro } {
	global simulationControl
	if {$smacro=="off"} {
		set simulationControl(modehomopolymerization) 0
	} else {
		set simulationControl(modehomopolymerization) 1
	} 
}
#SetCopolymerization
#SetCopolymerization on / off
proc SetCopolymerization { scopo } {
	global simulationControl
	if {$scopo=="off"} {
		set simulationControl(modecopo) 0
	} else {
		set simulationControl(modecopo) 1
		set simulationControl(modemacro) 0
	} 
}

# InitSimulation
# InitSimulation molecules
proc InitSimulation { nMolecules } {
	global nReactions
	global Reaction
	global nSpecies
	global Species	
	global nRateConstants
	global Constant
	global server1
	global Temperature
	global time
	global timesteps
	global umrechnung 
	global simulationControl

	set debugm 0
	if {$debugm==1} {
		set cmdf [open "init.sim-doc" w]
	} else {
		set cmdf [open "init.sim" w]
	}
	if {$simulationControl(modehomopolymerization)==1} {
		puts $cmdf "calculateMacromolecules\t3"
	} else {
		if {$simulationControl(modecopo)==1} {
			puts $cmdf "calculateMacromolecules\t2"
		} else {
			if {$simulationControl(modemacro)==1} {
				puts $cmdf "calculateMacromolecules\t1"
			} else {
				puts $cmdf "calculateMacromolecules\t0"
			}
		}
	}
	puts $cmdf [format "Species\t%u" $nSpecies]

	set csum 0.0
	for {set i 1} {$i<=$nSpecies} {incr i} {
		set csum [expr $csum + $Species($i,concentration)]
	}

	set umrechnung [expr $csum/$nMolecules]
	
	for {set i 1} {$i<=$nSpecies} {incr i} {
		set mole($i) [expr $nMolecules*$Species($i,concentration)/$csum]
		set Species($i,mole0) $mole($i)
	}



	puts -nonewline "register species "
	set resf [open "result.0" w]
	puts $resf 0.0
	for {set i 1} {$i<=$nSpecies} {incr i} {
		
		if {$Species($i,macromolecular)>0} {
			set stype [expr 100 + $Species($i,macromolecular)]
		} else {
			if {$Species($i,type)=="Monomer"} {
				if {$Species($i,name)=="Styrene"} {
					# register monomer Styrene
					set stype 3
				} else {
					# register any monomer
					set stype 2
				}
				
			} else {
				
				if {$Species($i,type)=="Initiator"} {
					set stype 1
				} else {
					set stype 0
				}
			}
		}
		if {$debugm==1} {
			if {($stype==2)||($stype==3)} {
				puts $cmdf [format "%s:\t%u\t%u\t%.8e\t%.1f" $Species($i,name) $i $stype $mole($i)]
			} else {
				puts $cmdf [format "%s:\t%u\t%u" $Species($i,name) $i $stype $mole($i)]
			}
		} else {
			if {($stype==2)||($stype==3)} {
				puts $cmdf [format "%u\t%u\t%.8e" $i $stype $mole($i)]
			} else {
				if {$stype==1} {
					puts $cmdf [format "%u\t%u\t%.8e\t%.3f" $i $stype $mole($i) $Species($i,f)]
				} else {
					puts $cmdf [format "%u\t%u\t%.8e\t%.3f" $i $stype $mole($i) 0.0]
				}
			}
		}
		#puts $resf [format "%u\t%u" $i [expr int($mole($i))]]
		puts $resf [format "%u\t%e" $i $mole($i)]
		
	}
	#close $resf

	puts "ok"
	
	puts -nonewline "register rate constants "
	for {set i 1} {$i<=$nRateConstants} {incr i} {
		set Constant($i,used) 0
		set Constant($i,order) 0
		set Constant($i,cf) 1.0
	}
	for {set i 1} {$i<=$nReactions} {incr i} {
		set ik [checkDefinedRateConstant $Reaction($i,constant)]
		set Constant($ik,used) 1
		set rorder $Reaction($i,nEducts)
		set rtype 0
		switch -exact $Reaction($i,type) {
			"Termination2P" { set rtype 3 }
			"TerminationD" { set rtype 9 }
		}
		set cfaktor 1.0
		if {$rorder==2} {
			if {$Reaction($i,educt1) == $Reaction($i,educt2)} {
				set cfaktor 2.0
			} 
		} 
		
		set Constant($ik,cf) $cfaktor 
		set Constant($ik,order) $rorder
	}
	set usedConstants 0
	for {set i 1} {$i<=$nRateConstants} {incr i} {
		if {$Constant($i,used)!=0} {
			if {$Constant($i,type)=="tempIndep"} {
				set k $Constant($i,value)
				for {set j 2} {$j<=$Constant($i,order)} {incr j} {
					set k [expr $k*$umrechnung]
				}
				set Constant($i,MCvalue) [expr $k*$Constant($i,cf)]
			} else {
				set k0 $Constant($i,k0)
				set ea $Constant($i,ea)
				set k [expr $k0*exp(-1.0*$ea/(273.15+$Temperature))]
				for {set j 2} {$j<=$Constant($i,order)} {incr j} {
					set k [expr $k*$umrechnung]
				}
				set Constant($i,MCvalue) [expr $k*$Constant($i,cf)]
			}
			incr usedConstants

		}
	}
	puts $cmdf [format "MCRates\t%u" $usedConstants]	
	for {set i 1} {$i<=$nRateConstants} {incr i} {
		if {$Constant($i,used)!=0} {
			if {$debugm==1} {
				puts $cmdf [format "%s\t%u\t%.12e" $Constant($i,name) $i $Constant($i,MCvalue)]
			} else {
				puts $cmdf [format "%u\t%.12e" $i $Constant($i,MCvalue)]
			}
		}
	}
	close $resf
	puts "ok"
	puts -nonewline "register reactions "
	puts $cmdf [format "Reactions\t%u" $nReactions]	
	for {set i 1} {$i<=$nReactions} {incr i} {
		if {$debugm==1} {
			puts -nonewline $cmdf [format "%s\t%u" $Reaction($i,type) $i]
		} else {
			puts -nonewline $cmdf [format "%u" $i]
		}
		set rtype 0
		switch -exact $Reaction($i,type) {
			"InitiatorDecomposition" { set rtype 1 }
			"Propagation" { set rtype 2 }
			"Termination2P" { set rtype 3 }
			"Transfer2monomer" { set rtype 4 }
			"ThermalInitiation" { set rtype 5 }
			"Initiation" { set rtype 6 }
			"Transfer_PL-P" { set rtype 7 }
			"Transfer_PL-PL" { set rtype 7 }
			"Transfer_P-PL" { set rtype 7 }
			"TerminationD" { set rtype 8 }
			"ChainAddition" { set rtype 9 }
			"ChainFragmentation" { set rtype 10 }
			"InitiationCopolymer11" {set rtype 21 }
			"InitiationCopolymer12" {set rtype 22 }
			"InitiationCopolymer21" {set rtype 23 }
			"InitiationCopolymer22" {set rtype 24 }
			"PropagationCopolymer1" {set rtype 25 }
			"PropagationCopolymer2" {set rtype 26 }
			"TransferCopolymer_PL-P" {set rtype 27 }
			"TransferCopolymer_P-PL" {set rtype 27 }
			"TerminationDispCopolymer" {set rtype 28 }
			"TerminationCombCopolymer" {set rtype 29 }
			
		}
		puts -nonewline $cmdf [format "\t%u" $rtype]
		set ik [checkDefinedRateConstant $Reaction($i,constant)]
		puts -nonewline $cmdf [format "\t%u" $ik]
		puts -nonewline $cmdf [format "\t%u" $Reaction($i,nEducts)]
		for {set j 1} {$j<=$Reaction($i,nEducts)} {incr j} {
			set side [checkDefinedSpecies $Reaction($i,educt$j)]
			puts -nonewline $cmdf [format "\t%u" $side]
		}
		puts -nonewline $cmdf [format "\t%u" $Reaction($i,nProducts)]	
		for {set j 1} {$j<=$Reaction($i,nProducts)} {incr j} {
			set sidp [checkDefinedSpecies $Reaction($i,product$j)]
			puts -nonewline $cmdf [format "\t%u" $sidp]
		}
		puts $cmdf ""
	}
	close $cmdf
	puts "ok" 
	set time 0.0
	set timesteps 0
	set rf [open "result.init" w]
	puts $rf [format "%.8e" $umrechnung]
	for {set i 1} {$i<=$nSpecies} {incr i} {
		puts $rf [format "%u\t%s" $i $Species($i,name)]
	}
	close $rf
	initSim
}

# __________________________________________________________________
# 6. Simulation  control

# internal getConversion
proc getConversion {SpeciesIndex timestep} {
	global Species
	
	set resf [open "result.$timestep" r]
	set idcount 0
	gets $resf zeile1
	while {[gets $resf zeile1]!= -1} {
		incr idcount
		set lzeile1 [split $zeile1]
		if {$SpeciesIndex == $idcount} {
			set lzeile1 [split $zeile1]
			set mole [lindex $lzeile1 1]
			set conversion [expr ($Species([expr $SpeciesIndex-0],mole0) - $mole) / $Species([expr $SpeciesIndex-0],mole0)]
			close $resf
			return $conversion
		}
	}
	close $resf
	return 0.0
}

# Simulation
# Simulation time (s)
proc Simulation {endtime} {
	global server1
	global nSpecies
	global Species	
	global Polymers
	global timesteps	
	global timestepValues
	global simulationControl
	global nReactions
	global Reaction
	global Temperature
	global umrechnung 
	global time	
	global debug
	
	set commandcount 0
	set fcount 0
	set resf [open "result.$fcount" r]
	gets $resf zeile1
	set lzeile1 [split $zeile1]
	set timestep($fcount) [lindex $lzeile1 0]
	for {set i 1} {$i<=$nSpecies} {incr i} {
		gets $resf zeile1
		set lzeile1 [split $zeile1]
		set mole($fcount,$i) [lindex $lzeile1 1]
	}
	close $resf
	while {$time<$endtime} {
		set time [expr $time + $simulationControl(dt)]
		if {$time>$endtime} {set time $endtime} 
		
		if {$simulationControl(hamielec)==1} {
			if {$timesteps>0} {
				set conversionstyrene [getConversion 1 $timesteps]
				hamielecModel $conversionstyrene
			}
		}
		if {$simulationControl(MA_Model)==1} {
			if {$timesteps>0} {
				set conversionma [getConversion 1 $timesteps]
				MA_Model $conversionma
			}
		}
		if {$simulationControl(cleanupresults) == 1} {
			file delete -force "result.$fcount"
		}
		incr commandcount
		gotoTime $commandcount $time
		puts "time: $time s"
		incr timesteps
		# analyze results
		incr fcount
		set resf [open "result.$fcount" r]
		gets $resf zeile1
		set lzeile1 [split $zeile1]
		set timestep($fcount) [lindex $lzeile1 0]
		for {set i 1} {$i<=$nSpecies} {incr i} {
			gets $resf zeile1
			set lzeile1 [split $zeile1]
			set mole($fcount,$i) [lindex $lzeile1 1]
			if {$Species($i,macromolecular)>0} {
				set pf [open "polymer.$fcount.$i" r]
				gets $pf  zeile1
				set lzeile1 [split $zeile1]
				if {[lindex $lzeile1 1]<100} {
					set Pn($fcount,$i) 0.0
					set Pw($fcount,$i) 0.0
				} else {
					gets $pf  zeile1
					set lzeile1 [split $zeile1] 
					set Pn($fcount,$i) [lindex $lzeile1 1]
					set Pw($fcount,$i) [lindex $lzeile1 3]
				}
				close $pf
				file rename -force "polymer.$fcount.$i" "$Species($i,name).$timestep($fcount).cld"
			}
		} 
		close $resf
	} 
	if {$simulationControl(cleanupresults) == 1} {
			file delete -force "result.$fcount"
	}
	for {set i 1} {$i<=$nSpecies} {incr i} {
		set cf [open "c$Species($i,name).dat" w]
		puts "write c$Species($i,name)"
		for {set j 0} {$j<=$fcount} {incr j} {
			puts $cf [format "%f\t%.8e\t%.8e" $timestep($j) [expr $mole($j,$i)*$umrechnung] $mole($j,$i)]
		}
		close $cf
		if {$Species($i,macromolecular)>0} {
			set cf [open "$Species($i,name).polymer.dat" w]
			for {set j 1} {$j<=$fcount} {incr j} {
				puts $cf [format "%f\t%.1f\t%.1f" $timestep($j) $Pn($j,$i) $Pw($j,$i)]
			}
			close $cf
		}
	}
	
}

